@@ -548,16 +548,16 @@ static int minimize_candidates(fits *image, star_finder_params *sf, starc *candi
 		/* FILL z */
 		if (image->type == DATA_USHORT) {
 			for (jj = 0, j = y - R; j <= y + R; j++, jj++) {
-				for (ii = 0, i = x - R; i <= x + R;
-						i++, ii++) {
-					gsl_matrix_set(z, ii, jj, (double)image_ushort[j][i]);
+				for (ii = 0, i = x - R; i <= x + R; i++, ii++) {
+					gsl_matrix_set(z, jj, ii, (double)image_ushort[j][i]);
+
 				}
 			}
 		} else {
 			for (jj = 0, j = y - R; j <= y + R; j++, jj++) {
-				for (ii = 0, i = x - R; i <= x + R;
-						i++, ii++) {
-					gsl_matrix_set(z, ii, jj, (double)image_float[j][i]);
+				for (ii = 0, i = x - R; i <= x + R; i++, ii++) {
+					gsl_matrix_set(z, jj, ii, (double)image_float[j][i]);
+
 				}
 			}
 		}
@@ -569,8 +569,8 @@ static int minimize_candidates(fits *image, star_finder_params *sf, starc *candi
 				//fwhm_to_arcsec_if_needed(image, cur_star);	// should we do this here?
 				cur_star->layer = layer;
 				int result_index = g_atomic_int_add(&nbstars, 1);
-				cur_star->xpos = (x - R) + cur_star->x0 - 1.0;
-				cur_star->ypos = (y - R) + cur_star->y0 - 1.0;
+				cur_star->xpos = (x - R) + cur_star->x0 - 0.5;
+				cur_star->ypos = (y - R) + cur_star->y0 - 0.5;
 				results[result_index] = cur_star;
 				//fprintf(stdout, "%03d: %11f %11f %f\n",
 				//		result_index, cur_star->xpos, cur_star->ypos, cur_star->mag);
@@ -637,7 +637,7 @@ psf_star *add_star(fits *fit, int layer, int *index) {
 	} else {
 		if (i < MAX_STARS) {
 			result->xpos = result->x0 + com.selection.x - 0.5;
-			result->ypos = com.selection.y + com.selection.h - result->y0 - 0.5;
+			result->ypos = com.selection.y + com.selection.h - result->y0 + 0.5;
 			com.stars[i] = result;
 			com.stars[i + 1] = NULL;
 			*index = i;
