@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -44,6 +44,7 @@
 #include "gui/dialogs.h"
 #include "gui/progress_and_log.h"
 #include "io/image_format_fits.h"
+#include "io/sequence.h"
 #include "sorting.h"
 #include "statistics.h"
 #include "statistics_float.h"
@@ -465,13 +466,12 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int layer, rectan
 	}
 }
 
-int compute_means_from_flat_cfa_ushort(fits *fit, float mean[4]) {
-	int row, col, c, i = 0;
+int compute_means_from_flat_cfa_ushort(fits *fit, double mean[36]) {
+	int row, col, c, i[36] = {0};
 	WORD *data;
 	unsigned int width, height;
 	unsigned int startx, starty;
 
-	mean[0] = mean[1] = mean[2] = mean[3] = 0.0;
 	data = fit->data;
 	width = fit->rx;
 	height = fit->ry;
@@ -482,31 +482,27 @@ int compute_means_from_flat_cfa_ushort(fits *fit, float mean[4]) {
 	startx = width / 3;
 	starty = height / 3;
 
-	/* make sure it does start at the beginning of CFA pattern */
-	startx = startx % 2 == 0 ? startx : startx + 1;
-	starty = starty % 2 == 0 ? starty : starty + 1;
-
 	siril_debug_print("Computing stat in (%d, %d, %d, %d)\n", startx, starty,
 			width - 1 - startx, height - 1 - starty);
 
-	/* Compute mean of each channel */
-	for (row = starty; row < height - 1 - starty; row += 2) {
-		for (col = startx; col < width - 1 - startx; col += 2) {
-			mean[0] += (float) data[col + row * width];
-			mean[1] += (float) data[1 + col + row * width];
-			mean[2] += (float) data[col + (1 + row) * width];
-			mean[3] += (float) data[1 + col + (1 + row) * width];
-			i++;
+	/* compute mean of each element in 6x6 blocks */
+	for (row = starty; row < height - starty; row++) {
+		for (col = startx; col < width - startx; col++) {
+			mean[(col % 6) + (row % 6) * 6] += (double) data[col + row * width];
+			i[(col % 6) + (row % 6) * 6]++;
 		}
 	}
 
-	for (c = 0; c < 4; c++) {
-		mean[c] /= (float) i;
+	for (c = 0; c < 36; c++) {
+		mean[c] /= (double) i[c];
 	}
 	return 0;
 }
 
-int compute_means_from_flat_cfa(fits *fit, float mean[4]) {
+/*  the mean is computed by adding data over and over in a long loop; 
+    but if the mean variable is a single-precision float, this results in
+    the accumulation of rounding errors. */
+int compute_means_from_flat_cfa(fits *fit, double mean[36]) {
 	if (fit->type == DATA_USHORT)
 		return compute_means_from_flat_cfa_ushort(fit, mean);
 	if (fit->type == DATA_FLOAT)
@@ -689,10 +685,15 @@ static int stat_image_hook(struct generic_seq_args *args, int o, int i, fits *fi
 	struct stat_data *s_args = (struct stat_data*) args->user;
 
 	for (int layer = 0; layer < fit->naxes[2]; layer++) {
-		imstats* stat = statistics(NULL, -1, fit, layer, &com.selection, s_args->option, TRUE);
+		/* we first check for data in cache */
+		imstats* stat = statistics(args->seq, i, NULL, layer, &s_args->selection, s_args->option, TRUE);
 		if (!stat) {
-			siril_log_message(_("Error: statistics computation failed.\n"));
-			return 1;
+			/* if no cache */
+			stat = statistics(args->seq, i, fit, layer, &s_args->selection, s_args->option, TRUE);
+			if (!stat) {
+				siril_log_message(_("Error: statistics computation failed.\n"));
+				return 1;
+			}
 		}
 
 		int new_index = i * s_args->seq->nb_layers;
@@ -764,8 +765,7 @@ static int stat_finalize_hook(struct generic_seq_args *args) {
 
 	int size = s_args->seq->nb_layers * args->nb_filtered_images;
 	GFile *file = g_file_new_for_path(s_args->csv_name);
-	GOutputStream* output_stream = (GOutputStream*) g_file_replace(file, NULL, FALSE,
-			G_FILE_CREATE_NONE, NULL, &error);
+	GOutputStream* output_stream = (GOutputStream*) g_file_replace(file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error);
 	g_free(s_args->csv_name);
 	if (output_stream == NULL) {
 		if (error != NULL) {
@@ -808,6 +808,7 @@ static int stat_finalize_hook(struct generic_seq_args *args) {
 	}
 
 	siril_log_message(_("Statistic file %s was successfully created.\n"), g_file_peek_path(file));
+	writeseqfile(args->seq);
 	g_object_unref(output_stream);
 	g_object_unref(file);
 	free_stat_list(s_args->list, size);
@@ -816,11 +817,51 @@ static int stat_finalize_hook(struct generic_seq_args *args) {
 	return 0;
 }
 
+static int stat_compute_mem_limit(struct generic_seq_args *args, gboolean for_writer) {
+	unsigned int MB_per_image, MB_avail, required;
+	int limit = compute_nb_images_fit_memory(args->seq, 1.0, FALSE, &MB_per_image, NULL, &MB_avail);
+
+	int is_color = args->seq->nb_layers == 3;
+	required = is_color ? MB_per_image * 4 / 3 : MB_per_image * 2;
+
+	if (limit > 0) {
+		int thread_limit = MB_avail / required;
+		if (thread_limit > com.max_thread)
+                        thread_limit = com.max_thread;
+		limit = thread_limit;
+
+		/* should not happen */
+		if (for_writer)
+			return 1;
+	}
+	if (limit == 0) {
+		gchar *mem_per_thread = g_format_size_full(required * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+		gchar *mem_available = g_format_size_full(MB_avail * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+
+		siril_log_color_message(_("%s: not enough memory to do this operation (%s required per image, %s considered available)\n"),
+				"red", args->description, mem_per_thread, mem_available);
+
+		g_free(mem_per_thread);
+		g_free(mem_available);
+	} else {
+#ifdef _OPENMP
+		siril_debug_print("Memory required per thread: %u MB, per image: %u MB, limiting to %d %s\n",
+				required, MB_per_image, limit, for_writer ? "images" : "threads");
+#else
+		if (!for_writer)
+			limit = 1;
+#endif
+	}
+	return limit;
+}
+
+
 void apply_stats_to_sequence(struct stat_data *stat_args) {
 	struct generic_seq_args *args = create_default_seqargs(stat_args->seq);
 	args->seq = stat_args->seq;
 	args->filtering_criterion = seq_filter_included;
 	args->nb_filtered_images = stat_args->seq->selnum;
+	args->compute_mem_limits_hook = stat_compute_mem_limit;
 	args->prepare_hook = stat_prepare_hook;
 	args->finalize_hook = stat_finalize_hook;
 	args->image_hook = stat_image_hook;
@@ -843,3 +884,4 @@ void on_menu_gray_stat_activate(GtkMenuItem *menuitem, gpointer user_data) {
 	siril_open_dialog("StatWindow");
 	set_cursor_waiting(FALSE);
 }
+
